@@ -115,6 +115,12 @@ struct {
     "Depth Textures",
     "CoglFramebuffers can be configured to render their depth buffer into "
     "a texture"
+  },
+  {
+    COGL_FEATURE_ID_PER_VERTEX_POINT_SIZE,
+    "Per-vertex point size",
+    "cogl_point_size_in can be used as an attribute to specify a per-vertex "
+    "point size"
   }
 };
 
@@ -141,6 +147,8 @@ get_winsys_name_for_id (CoglWinsysID winsys_id)
       return "EGL + KMS platform";
     case COGL_WINSYS_ID_EGL_ANDROID:
       return "EGL + Android platform";
+    case COGL_WINSYS_ID_EGL_MIR:
+      return "EGL + Mir platform";
     case COGL_WINSYS_ID_WGL:
       return "EGL + Windows WGL platform";
     case COGL_WINSYS_ID_SDL:
@@ -227,7 +235,11 @@ main (int argc, char **argv)
   const char *winsys_name;
   OutputState output_state;
 
+#ifdef COGL_HAS_EMSCRIPTEN_SUPPORT
+  ctx = cogl_sdl_context_new (SDL_USEREVENT, &error);
+#else
   ctx = cogl_context_new (NULL, &error);
+#endif
   if (!ctx) {
       fprintf (stderr, "Failed to create context: %s\n", error->message);
       return 1;
